
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include "test_sol.h"

#define LINE_BUF_LEN     2000

#define VERBOSE 1

#ifdef NOGETOPT
static char    name_buf[LINE_BUF_LEN];
static char    sol_buf[LINE_BUF_LEN];
//static int     opt;
#endif

#define ASCII_FORMAT    1
#define GZIP_FORMAT     2
#define BIN_FORMAT      3


#ifndef STR_ERR_UNKNOWN_LONG_OPT
# define STR_ERR_UNKNOWN_LONG_OPT   "%s: unrecognized option `--%s'\n"
#endif

#ifndef STR_ERR_LONG_OPT_AMBIGUOUS
# define STR_ERR_LONG_OPT_AMBIGUOUS "%s: option `--%s' is ambiguous\n"
#endif

#ifndef STR_ERR_MISSING_ARG_LONG
# define STR_ERR_MISSING_ARG_LONG   "%s: option `--%s' requires an argument\n"
#endif

#ifndef STR_ERR_UNEXPEC_ARG_LONG
# define STR_ERR_UNEXPEC_ARG_LONG   "%s: option `--%s' doesn't allow an argument\n"
#endif

#ifndef STR_ERR_UNKNOWN_SHORT_OPT
# define STR_ERR_UNKNOWN_SHORT_OPT  "%s: unrecognized option `-%c'\n"
#endif

#ifndef STR_ERR_MISSING_ARG_SHORT
# define STR_ERR_MISSING_ARG_SHORT  "%s: option `-%c' requires an argument\n"
#endif

#define STR_HELP_INSTANCE			\
  "  -i, --instance ARG    the instance file\n"

#define STR_HELP_SOLUTION			\
  "  -s, --solution ARG    the solution file\n"

#define STR_HELP_PROBLEM				\
  "  -p, --problem ARG     the problem -- see help\n"

#define STR_HELP_HELP						\
  "      --help            display this help text and exit\n"

#define STR_HELP				\
  STR_HELP_INSTANCE				\
  STR_HELP_SOLUTION				\
  STR_HELP_PROBLEM				\
  STR_HELP_HELP

/* Set to 1 if option --instance (-i) has been specified.  */
char opt_instance;

/* Set to 1 if option --solution (-s) has been specified.  */
char opt_solution;

/* Set to 1 if option --problem (-t) has been specified.  */
char opt_problem;

/* Set to 1 if option --help has been specified.  */
char opt_help;


/* Parse command line options.  Return index of first non-option argument,
   or -1 if an error is encountered.  */

/*************************************************************
 ************** utilities part ********************************
 ************************************************************/

void usage() {
  fprintf(stderr,"\nSolution Verifier for Weighted Clique\n\n");
  fprintf(stderr,"Usage: test_clique -i [INST_FILE] -s [SOL_FILE] -p [PROB_TYPE]\n");
  fprintf(stderr,"--help            -h  \t help: prints this information\n\n");
  fprintf(stderr,"Instance File Format: DIMACS file format. Nodes start from 1.\n");
  fprintf(stderr,"Solution File Format: nodes wrote in column. Nodes start from 1.\n");
  fprintf(stderr,"After each entry the character \\n (new line) has to be printed.\n");
  fprintf(stderr,"The Problem Type: clique 1\n");
  fprintf(stderr,"Report bugs to <marco@imada.sdu.dk>\n\n");
}

void checkOptions(const char *const program_name, const int argc, char **const argv)
{
  int r = parse_options(program_name, argc, argv );

  if (r<0)
    {
      fprintf(stderr,"test_clique: You must specify input files and type of problem\n");
      fprintf(stderr,"Try `test_clique --help' for more information.\n");
      exit(1);
    }
  if (opt_help)
    {
      usage();
      exit(1);
    }
  if (!opt_instance || !opt_solution) /* || !opt_problem)*/
    {
      fprintf(stderr,"test_clique: You must specify input files and type of problem\n");
      fprintf(stderr,"Try `test_clique --help' for more information.\n");
      exit(1);
    }
}

int parse_options (const char *const program_name, const int argc, char **const argv)
{
  static const char *const optstr__instance = "instance";
  static const char *const optstr__solution = "solution";
  static const char *const optstr__problem = "problem";
  static const char *const optstr__help = "help";
  int i = 0;
  opt_instance = 0;
  opt_solution = 0;
  opt_problem = 0;
  opt_help = 0;
  instance_name = 0;
  solution_name = 0;
  problem = 1;
  while (++i < argc)
    {
      const char *option = argv [i];
      if (*option != '-')
	return i;
      else if (*++option == '\0')
	return i;
      else if (*option == '-')
	{
	  const char *argument;
	  size_t option_len;
	  ++option;
	  if ((argument = strchr (option, '=')) == option)
	    goto error_unknown_long_opt;
	  else if (argument == 0)
	    option_len = strlen (option);
	  else
	    option_len = argument++ - option;
	  switch (*option)
	    {
	    case '\0':
	      return i + 1;
	    case 'h':
	      if (strncmp (option + 1, optstr__help + 1, option_len - 1) == 0)
		{
		  if (argument != 0)
		    {
		      option = optstr__help;
		      goto error_unexpec_arg_long;
		    }
		  opt_help = 1;
		  return i + 1;
		}
	      goto error_unknown_long_opt;
	    case 'i':
	      if (strncmp (option + 1, optstr__instance + 1, option_len - 1) == 0)
		{
		  if (argument != 0)
		    instance_name = argument;
		  else if (++i < argc)
		    instance_name = argv [i];
		  else
		    {
		      option = optstr__instance;
		      goto error_missing_arg_long;
		    }
		  opt_instance = 1;
		  break;
		}
	      goto error_unknown_long_opt;
	    case 'p':
	      if (strncmp (option + 1, optstr__problem + 1, option_len - 1) == 0)
		{
		  if (argument != 0)
		    problem = atoi(argument);
		  else if (++i < argc)
		    problem = atoi(argv [i]);
		  else
		    {
		      option = optstr__problem;
		      goto error_missing_arg_long;
		    }
		  opt_problem = 1;
		  break;
		}
	      goto error_unknown_long_opt;
	    case 's':
	      if (strncmp (option + 1, optstr__solution + 1, option_len - 1) == 0)
		{
		  if (argument != 0)
		    solution_name = argument;
		  else if (++i < argc)
		    solution_name = argv [i];
		  else
		    {
		      option = optstr__solution;
		      goto error_missing_arg_long;
		    }
		  opt_solution = 1;
		  break;
		}
	    default:
	    error_unknown_long_opt:
	      fprintf (stderr, STR_ERR_UNKNOWN_LONG_OPT, program_name, option);
	      return -1;
	    error_missing_arg_long:
	      fprintf (stderr, STR_ERR_MISSING_ARG_LONG, program_name, option);
	      return -1;
	    error_unexpec_arg_long:
	      fprintf (stderr, STR_ERR_UNEXPEC_ARG_LONG, program_name, option);
	      return -1;
	    }
	}
      else
	do
	  {
	    switch (*option)
	      {
	      case 'i':
		if (option [1] != '\0')
		  instance_name = option + 1;
		else if (++i < argc)
		  instance_name = argv [i];
		else
		  goto error_missing_arg_short;
		option = "\0";
		opt_instance = 1;
		break;
	      case 's':
		if (option [1] != '\0')
		  solution_name = option + 1;
		else if (++i < argc)
		  solution_name = argv [i];
		else
		  goto error_missing_arg_short;
		option = "\0";
		opt_solution = 1;
		break;
	      case 'p':
		if (option [1] != '\0')
		  problem = atoi(option + 1);
		else if (++i < argc)
		  problem = atoi(argv [i]);
		else
		  goto error_missing_arg_short;
		option = "\0";
		opt_problem = 1;
		break;
	      default:
		fprintf (stderr, STR_ERR_UNKNOWN_SHORT_OPT, program_name, *option);
		return -1;
	      error_missing_arg_short:
		fprintf (stderr, STR_ERR_MISSING_ARG_SHORT, program_name, *option);
		return -1;
	      }
	  } while (*++option != '\0');
    }
  return i;
}


long int **allocateMatrix( long int rows,  long int cols)
{
  long int i;
  long int **matrix;

  matrix = (long int **) malloc((rows+1) * sizeof(long int *));
  if (matrix == NULL)
    {
      fprintf(stderr,"Problems in memory allocation\n");
    }
  for (i=0; i < (rows+1) ;i++)
    {
      matrix[i] = (long int *) malloc((cols+1) * sizeof(long int));
      if (matrix[i] == NULL)
	{
	  fprintf(stderr,"Problems in memory allocation\n");
	}
      memset(matrix[i], 0, (cols+1)*sizeof(long int));
    }
  return matrix;
}

long int *allocateLongVector( long int size)
{
  long int *vector;

  vector = (long int *) malloc((size+1) * sizeof(long int ));
  if (vector == NULL)
    {
      fprintf(stderr,"Problems in memory allocation\n");
    }
  memset(vector, 0, (size+1)*sizeof(long int));
  return vector;
}

int *allocateIntVector( int size)
{
  int *vector;

  vector = (int *) malloc((size+1) * sizeof(int ));
  if (vector == NULL)
    {
      fprintf(stderr,"Problems in memory allocation\n");
    }
  memset(vector, 0, (size+1)*sizeof(int));
  return vector;
}


int int_compare (const void * a, const void * b)
{
  return ( *(int*)a - *(int*)b );
}


/*************************************************************
 **************** checker part ********************************
 ************************************************************/


void read_dimacs() {
  FILE *infile;
  int format = 0;
  char     buf [LINE_BUF_LEN];
  int num_of_edges = 0;
  int num_of_vertices = 0;
  int declared_edges = 0; 
  int v,w;
  long int ww;
  /* for doublechecking file definition*/
  
  if (strstr(instance_name, ".gz\0") != NULL)
    format = GZIP_FORMAT;
  else if (strstr(instance_name, ".b\0") != NULL)
    format = BIN_FORMAT;
  else
    format = ASCII_FORMAT;

  switch (format) {
  case ASCII_FORMAT: {
    printf("reading input in ASCII format...\n");
    infile = fopen(instance_name, "r");
    assert(infile != NULL);
    while (!feof(infile)) {
      buf[0] = '\0';
      fgets((char*) &buf, LINE_BUF_LEN, infile);
      switch (buf[0]) {
      case 'p': {
	char *tmp = strtok(buf, " "); 
	tmp = strtok(0, " "); 
	tmp = strtok(0, " ");
	num_of_vertices = atoi(tmp);
	tmp = strtok(0, " "); 
	declared_edges = atoi(tmp);
	weight = allocateLongVector(num_of_vertices);
	adjacency_matrix = allocateMatrix(num_of_vertices, num_of_vertices);
	for (v=0; v<num_of_vertices; v++)
	  weight[v]=1;
	break;
      }
      case 'e': {
	char *tmp;		
	tmp = strtok(buf, " "); 
	tmp = strtok(0, " "); 
	v = atoi(tmp) - 1;
	tmp = strtok(0, " "); 
	w = atoi(tmp) - 1;
	if (adjacency_matrix[v][w] == 0) {
	  num_of_edges++;
	  if (v > w) {
	    int h = v;
	    v = w;
	    w = h;
	  }
	}
	adjacency_matrix[v][w] = adjacency_matrix[w][v] = 1;
	break;
      }
      case 'n': {
	char *tmp;
	tmp = strtok(buf, " "); 
	tmp = strtok(0, " "); 
	v = atoi(tmp) - 1;
	tmp = strtok(0, " "); 
	ww = atol(tmp);
	weight[v]=ww;
	break;
      }
      case 'c':
	printf("%s", buf);
	break;
      case ' ':
      case '\n':
	break;
      default:
	break;
      }
    }
    
    fclose(infile);
      /*assert(num_of_edges == declared_edges);*/
    if (num_of_edges != declared_edges)
      printf("Warning: the instance has a number of edges different from those declared\n");
    break;

  }
  case GZIP_FORMAT: {
    printf("Read input in GZIP format...\n");
    exit(1);
    break;
  }
  case BIN_FORMAT: {
    printf("Read input in BIN format...\n");
    exit(1);
    break;
  }
  default:
    printf("A problem occurred in read_dimacs\n");
    break;
  }
  printf("done\n");
}

char checkSolution()
{
  int i,j;
  TOT_WEIGHT = 0;
  
  not_complete=0;
  for (i=0; i<SIZE; i++)
    {
      for (j=i+1; j<SIZE; j++)
	if (adjacency_matrix[solution[i]][solution[j]]!=1)
	  {
	    not_complete++;
	    break;
	  }
      TOT_WEIGHT+=weight[solution[i]];
    }
  if (not_complete>0)
    return FALSE;
  else 
    return TRUE;
}

void readSolution()
{
  FILE     *sol_file;
  int  count;
  int v;
  char   *p;

  char     buf [LINE_BUF_LEN];

  sol_file = fopen(solution_name, "r");
  assert(sol_file != NULL);

  printf("...reading solution file...");
  fgets((char*) &buf, LINE_BUF_LEN, sol_file);
  count = 0;
  while (!feof(sol_file)) {
    p = strtok(buf," ");
    do
      {
	if (*p!='\n')
	  {
	    v = atoi(p)-1;
	    solution = realloc(solution,(++count) * sizeof(int ));
	    solution[count-1]=v;
	  }
	p = strtok(0," ");
      } while(p != NULL);
    fgets((char*) &buf, LINE_BUF_LEN, sol_file);
  }
  SIZE=count;
  fclose(sol_file);
  printf("done\n");
}	





